@@ -1,6 +1,6 @@
 #!/usr/bin/perl -w
 #
-# Copyright (C) 2003  Pascal Giard
+# Copyright (C) 2003, 2004  Pascal Giard
 #
 # Author:             Pascal Giard <evilynux@yahoo.com>
 #
@@ -20,6 +20,7 @@
 
 use strict;
 
+use Carp;
 use File::stat;
 use POSIX qw(waitpid);
 use Getopt::Long;
@@ -54,6 +55,7 @@ use vars qw($NAME $xml_root $child_pid $mkisocmd
             @extra_mkisofs_options $extra_mkisofs_options
             @extra_mplayer_options $extra_mplayer_options
             @unwanted_mplayer_options $unwanted_mplayer_options
+            $filelist @real_filenames
            );
 
 $NAME="MoviXMaker-2";
@@ -116,6 +118,10 @@ if (!defined $opt_nogui)
 	error("The Gtk2::GladeXML perl module needs to be installed!\n".
 	      "Please refer to README file.\n".
 	      "-> Falling back to command line mode.\n");
+      } else {
+	# Will always work as Gtk2::GladeXML depends on libgtk2-perl
+	# which contains Gtk2::SimpleList
+	use Gtk2::SimpleList;
       }
     }
   }
@@ -244,11 +250,22 @@ sub on_button_create_released {
   $button->set_sensitive(FALSE);
   $destination->set_sensitive(FALSE);
   $srcdir->set_sensitive(FALSE);
+  $xml_root->get_widget ("cb_nocustomize")->set_sensitive(FALSE);
+  $xml_root->get_widget ("cb_noaddfiles")->set_sensitive(FALSE);
   $statusbar->set_status("Proceeding...");
 
   prepare_iso();
 }
 
+sub reenable {
+  $xml_root->get_widget ("button_create")->set_sensitive(TRUE);
+  $xml_root->get_widget ("cb_nocustomize")->set_sensitive(TRUE);
+  $xml_root->get_widget ("cb_noaddfiles")->set_sensitive(TRUE);
+  $destination->set_sensitive(TRUE);
+  $srcdir->set_sensitive(TRUE);
+}
+
+
 # The verbose_mkisofs_output window gets closed by the user
 sub on_fbbutton_released {
   $diag_feedback->hide();
@@ -261,6 +278,26 @@ sub on_option_change {
   $xml_root->get_widget ("prop_apply_btn")->set_sensitive (TRUE);
 }
 
+# The user is adding a file to the ISO
+sub on_add_file {
+  my ( $ok_button, $widget, $undef ) = @_;
+
+  foreach my $filename ( $widget->get_selections() ) {
+    push( @{$filelist->{'data'}}, $filename );
+    push @real_filenames, $filename;
+    print "Adding file: $filename\n"
+       if ($opt_debug);
+  }
+
+  $widget->hide();
+}
+
+# The user is adding a file to the ISO
+sub on_remove_file {
+  map( { delete( $filelist->{'data'}[$_] );
+	 delete( $real_filenames[$_] ) }
+       reverse( $filelist->get_selected_indices() ) );
+}
 ### end callbacks
 
 ###############################
@@ -274,7 +311,8 @@ sub using_installed_emovix {
 
   $srcpath = "$emovix_path/" unless (defined $srcpath);
   $installed_emovix = ($srcpath =~ /$emovix_path/) ? TRUE : FALSE;
-  print "Using installed eMoviX = $installed_emovix\n" if ($opt_debug);
+  print "Using installed eMoviX = $installed_emovix\n"
+    if ($opt_debug or $opt_nogui);
 }
 
 sub update_options {
@@ -586,6 +624,33 @@ sub prepare_interface {
   $diag_properties = $xml_root->get_widget ("diag_properties");
   $diag_properties->set_transient_for ($main_win); # diag_properties ber alles
 
+  # file selector dialog
+  my $diag_fileselection = $xml_root->get_widget ("diag_fileselection");
+  $diag_fileselection->set_select_multiple(TRUE);
+
+  # Filelist creation (files that will be add to the ISO image)
+  $filelist =
+    Gtk2::SimpleList->new (
+			   'Filename'    => 'text',
+			  );
+
+  # @{$slist->{data}} = (
+  #         [ 'TestFilm1.avi', '/home/share/films/' ],
+  #         [ 'TestFilm2.avi', '/home/share/films/' ],
+  # );
+
+  # Gtk2::SimpleList is derived from Gtk2::TreeView, so you can
+  # do anything you'd do to a treeview.
+  $filelist->set_rules_hint (TRUE);
+  $filelist->get_selection->set_mode ('multiple');
+  map { $_->set_resizable (TRUE) } $filelist->get_columns;
+
+  # packed into a scrolled window...
+  my $scrolled = $xml_root->get_widget ("scrolledwindow_addfiles");
+  $scrolled->add ($filelist);
+  # all in one go
+  $scrolled->show_all;
+
   #######################
   # Options specified at command line
   # are taken into account here
@@ -597,12 +662,6 @@ sub prepare_interface {
   $statusbar->set_status("Welcome!");
 }
 
-sub reenable {
-  $xml_root->get_widget ("button_create")->set_sensitive(TRUE);
-  $destination->set_sensitive(TRUE);
-  $srcdir->set_sensitive(TRUE);
-}
-
 # Display error dialog
 sub error {
   my ($err_msg) = @_;
@@ -998,11 +1057,30 @@ sub prepare_iso {
 	  update_options();
 
 	  # Did the user toggled the "Do not customize disc image" checkbox ?
-	  if ($xml_root->get_widget ("cb_nocustomize")->get_active) {
+	  if ($xml_root->get_widget ("cb_nocustomize")->get_active
+	     and $xml_root->get_widget ("cb_noaddfiles")->get_active) {
 	    # Directly creating the image, no customization
 	    create_iso() if (defined $srcpath);
+
+	  # User choose only to customize
+	  } elsif ($xml_root->get_widget ("cb_noaddfiles")->get_active) {
+	    $xml_root->get_widget ("btn_customize_next")->hide;
+	    $xml_root->get_widget ("btn_customize_ok")->show;
+	    $xml_root->get_widget ("diag_customize")->show()
+	      if (defined $srcpath);
+
+	  # User choose only to add files
+	  } elsif ($xml_root->get_widget ("cb_nocustomize")->get_active) {
+	    $xml_root->get_widget ("btn_addfiles_previous")->hide;
+	    $xml_root->get_widget ("diag_addfiles")->show()
+	      if (defined $srcpath);
+
+	  # User choose to customize disc image and add files
 	  } else {
 	    # Let's choose the options
+	    $xml_root->get_widget ("btn_customize_ok")->hide;
+	    $xml_root->get_widget ("btn_customize_next")->show;
+	    $xml_root->get_widget ("btn_addfiles_previous")->show;
 	    $xml_root->get_widget ("diag_customize")->show()
 	      if (defined $srcpath);
 	  }
@@ -1023,6 +1101,10 @@ sub create_tmp_folder {
   if ( -e $tmpDir ) {
     chdir  "$tmpDir";
 
+    foreach my $filename (@real_filenames) {
+      system "ln -s $filename ." if ($filename);
+    }
+
     # Include the movies that were copied into the src/ dir
     system "ln -s ${srcpath}${src}/* .";
 
@@ -1405,7 +1487,7 @@ Pascal Giard, E<lt>evilynux@yahoo.comE<gt>
 
 =head1 COPYRIGHT
 
-Copyright (C) 2003 Pascal Giard E<lt>evilynux@yahoo.comE<gt>
+Copyright (C) 2003, 2004 Pascal Giard E<lt>evilynux@yahoo.comE<gt>
 
 This package is free software; you can redistribute it and/or
 modify it under the same terms as Perl itself.
